// @ts-check
const { join } = require('path');
const { readJSON } = require('fs-extra');
const defaultFailBuild = (message, { error }) => {
    throw new Error(`${message}\n${error && error.stack}`);
};
const { HANDLER_FUNCTION_NAME, ODB_FUNCTION_NAME, HIDDEN_PATHS } = require('../constants');
const ODB_FUNCTION_PATH = `/.netlify/functions/${ODB_FUNCTION_NAME}`;
const HANDLER_FUNCTION_PATH = `/.netlify/functions/${HANDLER_FUNCTION_NAME}`;
const CATCH_ALL_REGEX = /\/\[\.{3}(.*)](.json)?$/;
const OPTIONAL_CATCH_ALL_REGEX = /\/\[{2}\.{3}(.*)]{2}(.json)?$/;
const DYNAMIC_PARAMETER_REGEX = /\/\[(.*?)]/g;
const getNetlifyRoutes = (nextRoute) => {
    let netlifyRoutes = [nextRoute];
    // If the route is an optional catch-all route, we need to add a second
    // Netlify route for the base path (when no parameters are present).
    // The file ending must be present!
    if (OPTIONAL_CATCH_ALL_REGEX.test(nextRoute)) {
        let netlifyRoute = nextRoute.replace(OPTIONAL_CATCH_ALL_REGEX, '$2');
        // When optional catch-all route is at top-level, the regex on line 19 will
        // create an empty string, but actually needs to be a forward slash
        if (netlifyRoute === '')
            netlifyRoute = '/';
        // When optional catch-all route is at top-level, the regex on line 19 will
        // create an incorrect route for the data route. For example, it creates
        // /_next/data/%BUILDID%.json, but NextJS looks for
        // /_next/data/%BUILDID%/index.json
        netlifyRoute = netlifyRoute.replace(/(\/_next\/data\/[^/]+).json/, '$1/index.json');
        // Add second route to the front of the array
        netlifyRoutes.unshift(netlifyRoute);
    }
    // Replace catch-all, e.g., [...slug]
    netlifyRoutes = netlifyRoutes.map((route) => route.replace(CATCH_ALL_REGEX, '/:$1/*'));
    // Replace optional catch-all, e.g., [[...slug]]
    netlifyRoutes = netlifyRoutes.map((route) => route.replace(OPTIONAL_CATCH_ALL_REGEX, '/*'));
    // Replace dynamic parameters, e.g., [id]
    netlifyRoutes = netlifyRoutes.map((route) => route.replace(DYNAMIC_PARAMETER_REGEX, '/:$1'));
    return netlifyRoutes;
};
exports.generateRedirects = async ({ netlifyConfig, basePath, i18n }) => {
    const { dynamicRoutes } = await readJSON(join(netlifyConfig.build.publish, 'prerender-manifest.json'));
    const redirects = [];
    netlifyConfig.redirects.push(...HIDDEN_PATHS.map((path) => ({
        from: `${basePath}${path}`,
        to: '/404.html',
        status: 404,
        force: true,
    })));
    const dynamicRouteEntries = Object.entries(dynamicRoutes);
    dynamicRouteEntries.sort((a, b) => a[0].localeCompare(b[0]));
    dynamicRouteEntries.forEach(([route, { dataRoute, fallback }]) => {
        // Add redirects if fallback is "null" (aka blocking) or true/a string
        if (fallback === false) {
            return;
        }
        redirects.push(...getNetlifyRoutes(route), ...getNetlifyRoutes(dataRoute));
    });
    // Needed only for /_next/static
    const i18nSplat = i18n ? '/:locale' : '';
    // This is only used in prod, so dev uses `next dev` directly
    netlifyConfig.redirects.push({ from: `${basePath}${i18nSplat}/_next/static/*`, to: `${i18nSplat}/static/:splat`, status: 200 }, {
        from: `${basePath}/*`,
        to: HANDLER_FUNCTION_PATH,
        status: 200,
        force: true,
        conditions: { Cookie: ['__prerender_bypass', '__next_preview_data'] },
    }, ...redirects.map((redirect) => ({
        from: `${basePath}${redirect}`,
        to: ODB_FUNCTION_PATH,
        status: 200,
    })), { from: `${basePath}/*`, to: HANDLER_FUNCTION_PATH, status: 200 });
};
exports.getNextConfig = async function getNextConfig({ publish, failBuild = defaultFailBuild }) {
    try {
        const { config, appDir } = await readJSON(join(publish, 'required-server-files.json'));
        if (!config) {
            return failBuild('Error loading your Next config');
        }
        return { ...config, appDir };
    }
    catch (error) {
        return failBuild('Error loading your Next config', { error });
    }
};
exports.setIncludedFiles = ({ netlifyConfig, publish }) => {
    // Serverless functions need parts of build dist in runtime env
    ;
    [HANDLER_FUNCTION_NAME, ODB_FUNCTION_NAME].forEach((functionName) => {
        if (!netlifyConfig.functions[functionName]) {
            netlifyConfig.functions[functionName] = {};
        }
        if (!netlifyConfig.functions[functionName].included_files) {
            netlifyConfig.functions[functionName].included_files = [];
        }
        netlifyConfig.functions[functionName].included_files.push(`${publish}/server/**`, `${publish}/serverless/**`, `${publish}/*.json`, `${publish}/BUILD_ID`);
    });
};
exports.setBundler = ({ netlifyConfig, target }) => {
    var _a;
    if (target === 'serverless') {
        // Any bundler works with serverless
        return;
    }
    // Not quite such a simple a check, as we need to check both the default bundler
    // and whether any of the individual functions are using the esbuild bundler.
    const handlers = [HANDLER_FUNCTION_NAME, ODB_FUNCTION_NAME];
    const defaultIsEsbuild = Boolean(((_a = netlifyConfig.functions['*']) === null || _a === void 0 ? void 0 : _a.node_bundler) === 'esbuild');
    const handlerBundlers = handlers.map((func) => { var _a; return (_a = netlifyConfig.functions[func]) === null || _a === void 0 ? void 0 : _a.node_bundler; });
    if ((defaultIsEsbuild && !handlerBundlers.every((bundler) => bundler === 'zisi')) ||
        handlerBundlers.includes('esbuild')) {
        console.warn(`esbuild is not supported for target=${target}. Setting to "zisi"`);
    }
    handlers.forEach((func) => {
        var _a;
        (_a = netlifyConfig.functions)[func] || (_a[func] = {});
        netlifyConfig.functions[func].node_bundler = 'zisi';
    });
};
